from gturtle import *
from dbtable import *
import time

# ---------------- Database management -------------
def makeReservation():
    # Get all reserved seats
    reserved = []
    reservation = DbTable()
    reservation.restore("casino")
    for row in reservation.select():
        if row[1] == 'Y':
            reserved.append(row[0])
    # Check if some selected seats are already reserved
    conflict = []
    for seat in selected:
        if seat in reserved:
            conflict.append(seat)
    if conflict != []:
        return conflict    
    # No conflict-> make reservation
    for seat in selected:
        reservation.update(seatnum = seat, _booked = 'Y')   
    reservation.save("casino")
    return []

# --------------- Callback executors ------------
def apply():
    if selected == []:
        setStatusText("Please make a seat selection.")
        return
    taken = makeReservation()
    if taken == []:
        setStatusText("Your reservation is confirmed. All reservations updated.")
        renew()
    else:
        setStatusText("Seat(s) " + str(taken) + " already taken. Left-click to deselect.")

def renew():
    reservation = DbTable()
    reservation.restore("casino")
    for row in reservation.select():
        if row[1] == 'Y':
            occupied.append(row[0])
    for seat in range(1, 16):
        setPos(toSeatPos(seat))
        if seat in occupied:
            drawImage("sprites/seat_2.png")
        else:
            drawImage("sprites/seat_0.png")
    selected.clear() 
# -------------- Mouse callback --------------------
def onMouseHit(x, y):
    if isRightMouseButton():
        apply()
        return
    if  y < -20 or y > 20 or x < -300 or x > 300:
        return
    seat = toSeat(x)
    setPos(toSeatPos(seat))
    if seat in occupied:
        setStatusText("This seat is occupied.")
        return
    if seat not in selected:
        drawImage("sprites/seat_1.png")
        selected.append(seat)
    else: 
        drawImage("sprites/seat_0.png")
        selected.remove(seat)

# ----------------- Coordinate convertions ---------
def toSeatPos(seat):
    return -320 + seat * 40, 0

def toLabelPos(seat):
    return -322 + seat * 40, -40

def toSeat(x):
    return int((x + 300) / 40 + 1)
        
# --------------- main -----------------------------        
selected = []
occupied = []
makeTurtle(mouseHit = onMouseHit)
setTitle("Multiuser Seat Reservation")
addStatusBar(20)
ht()
pu()
for seat in range(1, 16):
    setPos(toSeatPos(seat))
    drawImage("sprites/seat_0.png")
    setPos(toLabelPos(seat))
    label(str(seat), adjust = "c")

renew()
setStatusText("Left-click to select a free seat. Right-click to apply for a seat reservation.") 
while not isDisposed():
    time.sleep(1)
