# Mqtt_Horizon.py

from gpanel import *
from math import *
from paho.mqtt import subscribe

def rotate(pt, angle):
    _x = pt[0] * cos(angle) + pt[1] * sin(angle)
    _y = -pt[0] * sin(angle) + pt[1] * cos(angle)
    return (_x, _y)

def drawHorizon(pitch, roll_deg):
    lineWidth(2)
    # draw horizon area
    a = r * sin(radians(pitch))
    roll = radians(roll_deg)
    pt1 = (-sqrt(r * r - a * a), -a)
    pt2 = (sqrt(r * r - a * a), -a)
    # fill point lower area
    ptf = (0, -r + 2)
    # Rotate coordinate system now
    _pt1 = rotate(pt1, roll)
    _pt2 = rotate(pt2, roll)
    _ptf = rotate(ptf, roll)
    line(_pt1, _pt2)
    fill(_ptf, "white", "chocolate")
    # fill point upper area
    ptf = (0, r - 2)
    _ptf = rotate(ptf, roll)
    fill(_ptf, "white", "cornflower")

    # draw roll arrow
    lineWidth(2)
    setColor("black")
    pt1 = (0, r - 20)
    pt2 = (0, r) 
    _pt1 = rotate(pt1, roll)
    _pt2 = rotate(pt2, roll)
    arrow(_pt1, _pt2, 10)
    
    # draw aircraft
    lineWidth(3)
    setColor("black")
    pt1 = (-r / 2, 0)
    pt2 = (-10, 0)
    pt3 = (0, -10)
    pt4 = (10, 0)
    pt5 = (r / 2, 0)
    line(pt1, pt2)
    line(pt2, pt3)
    line(pt3, pt4)
    line(pt4, pt5)
    s = "pitch: %5d" %pitch
    text(-140, -140, s)    
    s = "roll: %5d" %roll_deg
    text(80, -140, s)    

def drawInstrument(pitch, roll):
    clear()
    rectangle(300, 300)
    setColor("darkgray")
    pos(0, 0)
    fillCircle(r + 10)
    setColor("white")
    fillCircle(r)
    setColor("darkgray")
    drawHorizon(pitch, roll)

def messageReceived(client, userdata, message):
    global oldRoll
    roll = int(message.payload)
    if roll != oldRoll:
        drawInstrument(0, int(message.payload))
        repaint()
        oldRoll = roll

makeGPanel(Size(400, 400))
title("Artificial Horizon")
window(-200, 200, -200, 200)
r = 100
enableRepaint(False)
oldRoll = 0

#host = "m2m.eclipse.org"
host = "broker.hivemq.com"
topic = "/ch/roll"
subscribe.callback(messageReceived, topic, hostname = host)


